@@ -115,15 +115,15 @@ typedef struct _SpecialOption {
 } SpecialOption;
 
 const SpecialOption specialOptions[] = {
-    {"run_key", "core", FALSE,
+    {"run_key", "gnomecompat", FALSE,
      METACITY "/global_keybindings/panel_run_dialog", OptionKey},
-    {"main_menu_key", "core", FALSE,
+    {"main_menu_key", "gnomecompat", FALSE,
      METACITY "/global_keybindings/panel_main_menu", OptionKey},
-    {"run_command_screenshot_key", "core", FALSE,
+    {"run_command_screenshot_key", "gnomecompat", FALSE,
      METACITY "/global_keybindings/run_command_screenshot", OptionKey},
-    {"run_command_window_screenshot_key", "core", FALSE,
+    {"run_command_window_screenshot_key", "gnomecompat", FALSE,
      METACITY "/global_keybindings/run_command_window_screenshot", OptionKey},
-    {"run_command_terminal_key", "core", FALSE,
+    {"run_command_terminal_key", "gnomecompat", FALSE,
      METACITY "/global_keybindings/run_command_terminal", OptionKey},
 
     {"toggle_window_maximized_key", "core", FALSE,
@@ -166,15 +166,18 @@ const SpecialOption specialOptions[] = {
      METACITY "/window_keybindings/activate_window_menu", OptionSpecial},
     {"mouse_button_modifier", NULL, FALSE,
      METACITY "/general/mouse_button_modifier", OptionSpecial},
+    /* integration of the Metacity's option to swap mouse buttons */
+    {"resize_with_right_button", NULL, FALSE,
+     METACITY "/general/resize_with_right_button", OptionSpecial},
 
     {"visual_bell", "fade", TRUE,
      METACITY "/general/visual_bell", OptionBool},
     {"fullscreen_visual_bell", "fade", TRUE,
      METACITY "/general/visual_bell_type", OptionSpecial},
 
-    {"next_key", "switcher", FALSE,
+    {"next_key", "staticswitcher", FALSE,
      METACITY "/global_keybindings/switch_windows", OptionKey},
-    {"prev_key", "switcher", FALSE,
+    {"prev_key", "staticswitcher", FALSE,
      METACITY "/global_keybindings/switch_windows_backward", OptionKey},
 
     {"toggle_sticky_key", "extrawm", FALSE,
@@ -182,54 +185,54 @@ const SpecialOption specialOptions[] = {
     {"toggle_fullscreen_key", "extrawm", FALSE,
      METACITY "/window_keybindings/toggle_fullscreen", OptionKey},
 
-    {"command0", "core", FALSE,
+    {"command0", "commands", FALSE,
      METACITY "/keybinding_commands/command_1", OptionString},
-    {"command1", "core", FALSE,
+    {"command1", "commands", FALSE,
      METACITY "/keybinding_commands/command_2", OptionString},
-    {"command2", "core", FALSE,
+    {"command2", "commands", FALSE,
      METACITY "/keybinding_commands/command_3", OptionString},
-    {"command3", "core", FALSE,
+    {"command3", "commands", FALSE,
      METACITY "/keybinding_commands/command_4", OptionString},
-    {"command4", "core", FALSE,
+    {"command4", "commands", FALSE,
      METACITY "/keybinding_commands/command_5", OptionString},
-    {"command5", "core", FALSE,
+    {"command5", "commands", FALSE,
      METACITY "/keybinding_commands/command_6", OptionString},
-    {"command6", "core", FALSE,
+    {"command6", "commands", FALSE,
      METACITY "/keybinding_commands/command_7", OptionString},
-    {"command7", "core", FALSE,
+    {"command7", "commands", FALSE,
      METACITY "/keybinding_commands/command_8", OptionString},
-    {"command8", "core", FALSE,
+    {"command8", "commands", FALSE,
      METACITY "/keybinding_commands/command_9", OptionString},
-    {"command9", "core", FALSE,
+    {"command9", "commands", FALSE,
      METACITY "/keybinding_commands/command_10", OptionString},
-    {"command10", "core", FALSE,
+    {"command10", "commands", FALSE,
      METACITY "/keybinding_commands/command_11", OptionString},
-    {"command11", "core", FALSE,
+    {"command11", "commands", FALSE,
      METACITY "/keybinding_commands/command_12", OptionString},
 
-    {"run_command0_key", "core", FALSE,
+    {"run_command0_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_1", OptionKey},
-    {"run_command1_key", "core", FALSE,
+    {"run_command1_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_2", OptionKey},
-    {"run_command2_key", "core", FALSE,
+    {"run_command2_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_3", OptionKey},
-    {"run_command3_key", "core", FALSE,
+    {"run_command3_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_4", OptionKey},
-    {"run_command4_key", "core", FALSE,
+    {"run_command4_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_5", OptionKey},
-    {"run_command5_key", "core", FALSE,
+    {"run_command5_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_6", OptionKey},
-    {"run_command6_key", "core", FALSE,
+    {"run_command6_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_7", OptionKey},
-    {"run_command7_key", "core", FALSE,
+    {"run_command7_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_8", OptionKey},
-    {"run_command8_key", "core", FALSE,
+    {"run_command8_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_9", OptionKey},
-    {"run_command9_key", "core", FALSE,
+    {"run_command9_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_10", OptionKey},
-    {"run_command10_key", "core", FALSE,
+    {"run_command10_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_11", OptionKey},
-    {"run_command11_key", "core", FALSE,
+    {"run_command11_key", "commands", FALSE,
      METACITY "/global_keybindings/run_command_12", OptionKey},
 
     {"rotate_to_1_key", "rotate", FALSE,
@@ -262,31 +265,6 @@ const SpecialOption specialOptions[] = {
     {"rotate_right_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_right", OptionKey},
 
-    {"plane_to_1_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_1", OptionKey},
-    {"plane_to_2_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_2", OptionKey},
-    {"plane_to_3_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_3", OptionKey},
-    {"plane_to_4_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_4", OptionKey},
-    {"plane_to_5_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_5", OptionKey},
-    {"plane_to_6_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_6", OptionKey},
-    {"plane_to_7_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_7", OptionKey},
-    {"plane_to_8_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_8", OptionKey},
-    {"plane_to_9_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_9", OptionKey},
-    {"plane_to_10_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_10", OptionKey},
-    {"plane_to_11_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_11", OptionKey},
-    {"plane_to_12_key", "plane", FALSE,
-     METACITY "/global_keybindings/switch_to_workspace_12", OptionKey},
-
     {"switch_to_1_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_1", OptionKey},
     {"switch_to_2_key", "vpswitch", FALSE,
@@ -376,11 +354,11 @@ const SpecialOption specialOptions[] = {
     {"rotate_right_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_right", OptionKey},
 
-    {"command_screenshot", "core", FALSE,
+    {"command_screenshot", "gnomecompat", FALSE,
      METACITY "/keybinding_commands/command_screenshot", OptionString},
-    {"command_window_screenshot", "core", FALSE,
+    {"command_window_screenshot", "gnomecompat", FALSE,
      METACITY "/keybinding_commands/command_window_screenshot", OptionString},
-    {"command_terminal", "core", FALSE,
+    {"command_terminal", "gnomecompat", FALSE,
      "/desktop/gnome/applications/terminal/exec", OptionString},
 
     {"current_viewport", "thumbnail", TRUE,
@@ -406,8 +384,8 @@ const SpecialOption specialOptions[] = {
 
 static CCSSetting *
 findDisplaySettingForPlugin (CCSContext *context,
-			     char       *plugin,
-			     char       *setting)
+			     const char *plugin,
+			     const char *setting)
 {
     CCSPlugin  *p;
     CCSSetting *s;
@@ -431,19 +409,35 @@ isIntegratedOption (CCSSetting *setting,
 
     for (i = 0; i < N_SOPTIONS; i++)
     {
-	if ((strcmp (setting->name, specialOptions[i].settingName) == 0) &&
-	    ((!setting->parent->name && !specialOptions[i].pluginName) ||
-	     (setting->parent->name && specialOptions[i].pluginName &&
-	      (strcmp (setting->parent->name,
-		       specialOptions[i].pluginName) == 0))) &&
-	    ((setting->isScreen && specialOptions[i].screen) ||
-	     (!setting->isScreen && !specialOptions[i].screen)))
+	const SpecialOption *opt = &specialOptions[i];
+
+	if (strcmp (setting->name, opt->settingName) != 0)
+	    continue;
+
+	if (setting->parent->name)
 	{
-	    if (index)
-		*index = i;
-	    return TRUE;
+	    if (!opt->pluginName)
+		continue;
+	    if (strcmp (setting->parent->name, opt->pluginName) != 0)
+		continue;
 	}
+	else
+	{
+	    if (opt->pluginName)
+		continue;
+	}
+
+	if (setting->isScreen && !opt->screen)
+	    continue;
+	if (!setting->isScreen && opt->screen)
+	    continue;
+
+	if (index)
+	    *index = i;
+
+	return TRUE;
     }
+
     return FALSE;
 }
 
@@ -553,8 +547,10 @@ gnomeValueChanged (GConfClient *client,
 	if (num < 0)
 	    break;
 
-	if (strcmp (specialOptions[num].settingName,
-		    "mouse_button_modifier") == 0)
+	if ((strcmp (specialOptions[num].settingName,
+		     "mouse_button_modifier") == 0) ||
+	    (strcmp (specialOptions[num].settingName,
+		    "resize_with_right_button") == 0))
 	{
 	    CCSSetting *s;
 
@@ -701,16 +697,19 @@ copyGconfValues (GConfEngine *conf,
 	    if (associate && schemaPath)
 		asprintf (&newSchema, "%s/%s", schemaPath, name + 1);
 
-	    value = gconf_engine_get_without_default (conf, key, NULL);
-	    if (value && newKey)
-	    {
-		if (newSchema)
-		    gconf_engine_associate_schema (conf, newKey,
-						   newSchema, NULL);
-		gconf_engine_set (conf, newKey, value, NULL);
+	    if (newKey && newSchema)
+		gconf_engine_associate_schema (conf, newKey, newSchema, NULL);
 
-		gconf_value_free (value);
+	    if (newKey)
+	    {
+		value = gconf_engine_get_without_default (conf, key, NULL);
+		if (value)
+		{
+		    gconf_engine_set (conf, newKey, value, NULL);
+		    gconf_value_free (value);
+		}
 	    }
+
 	    if (newSchema)
 		free (newSchema);
 	    if (newKey)
@@ -766,12 +765,10 @@ copyGconfRecursively (GConfEngine *conf,
 	    if (newSchema)
 		free (newSchema);
 
-	    if (to)
-	    {
-		if (newKey)
-		    free (newKey);
-	    }
-	    else
+	    if (newKey)
+		free (newKey);
+
+	    if (!to)
 		gconf_engine_remove_dir (conf, path, NULL);
 	}
 
@@ -963,6 +960,24 @@ getGnomeMouseButtonModifier(void)
     return modMask;
 }
 
+static unsigned int
+getButtonBindingForSetting (CCSContext   *context,
+			    const char   *plugin,
+			    const char   *setting)
+{
+    CCSSetting *s;
+
+    s = findDisplaySettingForPlugin (context, plugin, setting);
+    if (!s)
+	return 0;
+
+    if (s->type != TypeButton)
+	return 0;
+
+    return s->value->value.asButton.button;
+}
+
+
 static Bool
 readIntegratedOption (CCSContext *context,
 		      CCSSetting *setting,
@@ -1094,15 +1109,23 @@ readIntegratedOption (CCSContext *context,
 		      ((strcmp (settingName, "window_menu_button") == 0) &&
 		       (strcmp (pluginName, "core") == 0)))
 	    {
+		gboolean              resizeWithRightButton;
 		CCSSettingButtonValue button;
+
 		memset (&button, 0, sizeof (CCSSettingButtonValue));
 		ccsGetButton (setting, &button);
 
 		button.buttonModMask = getGnomeMouseButtonModifier ();
+		
+		resizeWithRightButton =
+		    gconf_client_get_bool (client, METACITY
+					   "/general/resize_with_right_button",
+					   &err);
+
 		if (strcmp (settingName, "window_menu_button") == 0)
-		    button.button = 3;
+		    button.button = resizeWithRightButton ? 2 : 3;
 		else if (strcmp (pluginName, "resize") == 0)
-		    button.button = 2;
+		    button.button = resizeWithRightButton ? 3 : 2;
 		else
 		    button.button = 1;
 
@@ -1345,10 +1368,10 @@ writeListValue (CCSSetting *setting,
 	break;
     case TypeFloat:
 	{
-	    float *item;
+	    gdouble *item;
 	    while (list)
 	    {
-		item = malloc (sizeof (float));
+		item = malloc (sizeof (gdouble));
 		if (item)
 		{
 		    *item = list->data->value.asFloat;
@@ -1418,28 +1441,25 @@ writeListValue (CCSSetting *setting,
 	g_slist_free (valueList);
 }
 
-static void
+static Bool
 setGnomeMouseButtonModifier (unsigned int modMask)
 {
     char   *modifiers, *currentValue;
     GError *err = NULL;
 
     modifiers = ccsModifiersToString (modMask);
-
     if (!modifiers)
-	modifiers = strdup ("");
-    if (!modifiers)
-	return;
+	return FALSE;
 
-    currentValue =
-	gconf_client_get_string(client,
-				METACITY "/general/mouse_button_modifier",
-				&err);
+    currentValue = gconf_client_get_string (client,
+					    METACITY
+					    "/general/mouse_button_modifier",
+					    &err);
     if (err)
     {
 	free (modifiers);
 	g_error_free (err);
-	return;
+	return FALSE;
     }
 
     if (!currentValue || (strcmp (currentValue, modifiers) != 0))
@@ -1450,6 +1470,8 @@ setGnomeMouseButtonModifier (unsigned int modMask)
 	g_free (currentValue);
 
     free (modifiers);
+
+    return TRUE;
 }
 
 static void
@@ -1462,7 +1484,7 @@ setButtonBindingForSetting (CCSContext   *context,
     CCSSetting            *s;
     CCSSettingButtonValue value;
 
-    s = findDisplaySettingForPlugin (context, (char*) plugin, (char*) setting);
+    s = findDisplaySettingForPlugin (context, plugin, setting);
     if (!s)
 	return;
 
@@ -1601,7 +1623,7 @@ writeIntegratedOption (CCSContext *context,
 		if (!ccsGetBool (setting, &clickToFocus))
 		    break;
 
-		newValue = clickToFocus ? "click" : "mouse";
+		newValue = clickToFocus ? "click" : "sloppy";
 		currentValue = gconf_client_get_string (client,
 							optionName, &err);
 
@@ -1620,15 +1642,45 @@ writeIntegratedOption (CCSContext *context,
 		       (strcmp (pluginName, "core") == 0)))
 	    {
 		unsigned int modMask;
+		Bool         resizeWithRightButton = FALSE;
+		gboolean     currentValue;
+
+		if ((getButtonBindingForSetting (context, "resize",
+						 "initiate_button") == 3) ||
+		    (getButtonBindingForSetting (context, "core",
+						 "window_menu_button") == 2))
+		{
+		     resizeWithRightButton = TRUE;
+		}
+
+		currentValue =
+		    gconf_client_get_bool (client, METACITY
+					   "/general/resize_with_right_button",
+					   &err);
+
+		if (!err && ((currentValue && !resizeWithRightButton) ||
+			     (!currentValue && resizeWithRightButton)))
+		{
+		    gconf_client_set_bool (client,
+					   METACITY
+					   "/general/resize_with_right_button",
+					   resizeWithRightButton, NULL);
+		}
 
 		modMask = setting->value->value.asButton.buttonModMask;
-		setGnomeMouseButtonModifier (modMask);
-		setButtonBindingForSetting (context, "move",
-					    "initiate_button", 1, modMask);
-		setButtonBindingForSetting (context, "resize",
-					    "initiate_button", 2, modMask);
-		setButtonBindingForSetting (context, "core",
-					    "window_menu_button", 3, modMask);
+		if (setGnomeMouseButtonModifier (modMask))
+		{
+		    setButtonBindingForSetting (context, "move",
+						"initiate_button", 1, modMask);
+		    setButtonBindingForSetting (context, "resize",
+						"initiate_button", 
+						resizeWithRightButton ? 3 : 2,
+						modMask);
+		    setButtonBindingForSetting (context, "core",
+						"window_menu_button",
+						resizeWithRightButton ? 2 : 3,
+						modMask);
+		}
 	    }
 	}
      	break;
@@ -1840,17 +1892,20 @@ checkProfile (CCSContext *context)
     else
 	currentProfile = strdup (profile);
 
-    if (strcmp (lastProfile, currentProfile) != 0)
+    if (!lastProfile || strcmp (lastProfile, currentProfile) != 0)
     {
 	char *pathName;
 
-	/* copy /apps/compiz tree to profile path */
-	asprintf (&pathName, "%s/%s", PROFILEPATH, lastProfile);
-	if (pathName)
+	if (lastProfile)
 	{
-	    copyGconfTree (context, "/apps/compiz", pathName,
-	    		   TRUE, "/schemas/apps/compiz");
-	    free (pathName);
+	    /* copy /apps/compiz tree to profile path */
+	    asprintf (&pathName, "%s/%s", PROFILEPATH, lastProfile);
+	    if (pathName)
+	    {
+		copyGconfTree (context, COMPIZ, pathName,
+			       TRUE, "/schemas" COMPIZ);
+		free (pathName);
+	    }
 	}
 
 	/* reset /apps/compiz tree */
